<?
/*	Project webdev
	DMC Ltd
	Copyright 1997-1999 DMC Ltd. All Rights Reserved.

	SUBSYSTEM: WebDev project management
	FILE: webdev.php3
	AUTHOR: Sandy Kydd (sandy@dmcsoft.com)

Actions supported, with parameters:

 * options <uri|dir>
 * mkcol   <uri|dir>
 * ls	   <uri|dir> <depth>
 * get	   <uri|dir>
 * put	   <uri|dir> <uploadfile>
 * delete  <uri|dir>
 * move	   <uri|dir> <moveto(uri)>
 * copy	   <uri|dir> <copyto(uri)>

<uri> will be converted into a <dir> location on server.
<dir> must be a valid location (allowed locations set in config file).
<depth> should be -1 for infinity

Return codes (DAV-like):
 102 Processing
 207 Multi-status
 422 Unprocessable entity
 423 Locked
 424 Failed dependency
 507 Insufficient storage

Standard return codes (a la HTTP)
 200 OK
 201 Created
 204 No Content
 400 Bad Request
 401 Authorization Required
 403 Forbidden
 404 Not Found
 405 Method Not Allowed
 
*/

Header("Content-type: text/plain");
require "config.php3";

class WdProject {
	var $uri, $dir;

	function WdProject ($uri, $dir) {
		global $HTTP_USER_AGENT;

		$this->uri = $uri;
		if ($uri)
			$this->dir = $this->uri2dir($uri);
		else
			$this->dir = $dir;
		if (!$this->dir) {
			print "400 Bad Request\n";
			exit;
		}
		$this->check_path($this->dir);
		$this->check_browser($HTTP_USER_AGENT);
	}
	function check_allowed ($val,$allow_type) {
		global $WD_INSTALL;

		// Check that val is allowed
		$ary = $WD_INSTALL[$allow_type];
		reset($ary);
		while (list($key) = each($ary)) {
			if (ereg("^$key",$val))
				return;
		}
		print "403 Forbidden ($val)\n";
		exit;
	}
	function check_path ($loc) {
		$this->check_allowed($loc,"AllowPaths");
	}
	function check_browser ($agent) {
		$this->check_allowed($agent,"AllowBrowsers");
	}
	function uri2dir ($uri) {
		global $WD_INSTALL;

		// Change uri into filesystem path
		// This works for apache, not sure how to do for other servers
		$dir = "";
		$req = apache_lookup_uri($uri);
		if (is_object($req))
			return $req->filename;

		//$dir = $WD_INSTALL["pubbase"]. $uri;
		return $dir;
	}

// THE ACTIONS
	function options () {
		print "OPTIONS, MKCOL, LS, GET, PUT, DELETE, MOVE, COPY\n";
	}
	function mkcol () {
		// Make new directory
		@mkdir($this->dir, 0775);
		if (is_dir($this->dir))
			print "201 Created ". $this->dir. "\n";
		else
			print "404 Not Found (". $this->dir. ")\n";
	}
	function ls () {
		global $depth;


		// List directory - should we also be sending
		// some file info (size/type/date)
		$dir = $this->dir;
		if (!is_dir($dir))
			return;
		$ary[] = ".";
		reset($ary);
		for ($i = current($ary); $i; $i = next($ary)) {
			$i = ereg_replace("^\./","",$i);
			$d = opendir("$dir/$i");
			while ($e = readdir($d)) {
				if (is_dir("$dir/$i/$e")) {
					if (!ereg("^\.",$e)) {
						if ($depth==-1)
							$ary[] = "$i/$e";
						else
							print "$e\r\n";
					}
				} else {
					$file = ereg_replace("^\./","","$i/$e");
					print "$file\r\n";
				}
			}
			closedir($d);
		}
	}
	function get () {
		// Return file
		if (is_file($this->dir))
			ReadFile($this->dir);
		else
			print "404 Not Found (". $this->dir. ")\n";
	}
	function put () {
		global $uploadfile;

		// Save file
		if (is_file($uploadfile)) {
			if (is_file($this->dir))
				@unlink($this->dir);
			@copy($uploadfile, $this->dir);
			@unlink($uploadfile);
			print "201 Created ". $this->dir. "\n";
		}
	}
	function delete () {
		if (is_file($this->dir)) {
			// Remove file
			@unlink($this->dir);
			print "200 OK (". $this->dir. ")\n";
		} elseif (is_dir($this->dir)) {
			// Remove directory
			@rmdir($this->dir);
			print "200 OK (". $this->dir. ")\n";
		} else {
			print "404 Not Found (". $this->dir. ")\n";
		}
	}
	function move () {
		global $moveto;

		// Move file/directory to new location
		// Need second uri/dir of where to move to
		$moveto = $this->uri2dir($moveto);
		$this->check_path($moveto);
		if (is_file($this->dir)) {
			if (is_file($moveto))
				unlink($moveto);
			copy($this->dir,$moveto);
			unlink($this->dir);
			print $this->dir. "\n";
		} else {
			print "404 Not Found (". $this->dir. ")\n";
		}
	}
	function copy () {
		global $copyto;

		// Copy directory contents (-uR) to new location
		// Need second uri/dir of where to copy to
		$copyto = $this->uri2dir($copyto);
		$this->check_path($copyto);

		@mkdir($copyto, 0775);
		if (!is_dir($copyto))
			return;
		print "$copyto\r\n";

		// Walk the tree directory
		$dir = $this->dir;
		$ary[] = ".";
		reset($ary);
		for ($i = current($ary); $i; $i = next($ary)) {
			$i = ereg_replace("^\./","",$i);
			// Make sure the copyto dir exists

			$d = opendir("$dir/$i");
			while ($e = readdir($d)) {
				$tmp = ereg_replace("^\./","","$i/$e");
				if (is_dir("$dir/$i/$e")) {
					if (!ereg("^\.",$e)) {
						$ary[] = "$i/$e";
						@mkdir("$copyto/$tmp", 0775);
						print "$copyto/$tmp\r\n";
					}
				} else {
					$from = "$dir/$tmp";
					$to = "$copyto/$tmp";
					// Copy this file if its newer
					if (filemtime($from) > filemtime($to)) {
						@copy($from, $to);
						print "$tmp\r\n";
					}
				}
			}
			closedir($d);
		}
	}
};

$proj = new WdProject($uri,$dir);
if ($action)
	$proj->$action();
?>
